#!/usr/bin/python
# vim: set ai sts=4 ts=8 sw=4 expandtab:
'''
Simplify Client Module
'''
import asyncore, socket, time, random, threading

class Cui:
    def notify(self, string):
        print string

class ClientBase(asyncore.dispatcher) :
    # System Methods
    def __init__(self, sock=None, ui=None) :
        asyncore.dispatcher.__init__(self)
        self.create_socket(socket.AF_INET, socket.SOCK_STREAM)

        self.max = 45
        self.count = 6
        self.round = 0
        
        self.ui = ui

        keywords = self.keywords = {}
        keywords['LOGIN'] = self.proc_login
        keywords['BUY'] = self.proc_buy

    def __call__(self):
        asyncore.loop(1.0)
    
    # Get/Set
    def getMax(self): return self.max
    def setMax(self, max): self.max = max
    def getCount(self): return self.count
    def setCount(self): self.count = count
    def getRound(self): return self.round
    def setRound(self): self.round = round

    # Internal
    def writable(self) :
        return 0

    def handle_connect(self):
        self.ui.notify('connect ok')

    def handle_read(self) :
        stream = self.recv(1024)
        packet_list = stream.splitlines()
        while packet_list:
            packet = packet_list.pop(0)
            (head, message) = packet.split(' ', 1)

            keywords = self.keywords
            if keywords.has_key(head):
                command = self.keywords[head]
                command(message)
            else:
                command = self.proc_dummy
                command(message)
    
    def handle_close(self):
        self.ui.die()
        self.close()

    # Command Procedure
    def proc_login(self, data):
        (extra, message) = data.split(' ', 1)

        if extra == 'OK':
            data = message.split()
            self.max = data[0]
            self.count = data[1]
            self.round = data[2]

            self.ui.notify('Login '+repr(data))
        else:
            self.ui.notify('Login failed')

    def proc_buy(self, data):
        self.ui.notify('Buy '+data)

    def proc_dummy(self, data):
        self.ui.notify(data)

    # External
    def login(self, name):
        self.send('LOGIN NULL '+name)

    def buy(self, slip):
        self.send('BUY NULL '+slip)

    def send(self, data):
        asyncore.dispatcher.send(self, data+'\n')

def getAnyNumbers(count, max) :
    numbers = []
    loop_cnt = count
    max = max

    while len(numbers) < loop_cnt :
        anything = random.randint(1, max)
        numbers.append(anything)
        if numbers.count(anything) >= 2 :
            numbers.remove(anything)

    return numbers

def getAnyName():
    name = ''
    for i in range(0,random.randint(3, 8)):
        name = name + chr(random.randint(ord('a'), ord('z')))
    return name

def listToString(list):
    string = repr(list)[1:-1]
    return string.replace(',', '')

def __test__() :
    core = ClientBase(ui=Cui())
    try:
        core.connect(('localhost', 19999))
        core_thr = threading.Thread(target=core)

        name = getAnyName()
        core_thr.start()

        core.login(name)
        time.sleep(0.2)

        try:
            while 1:
                number = getAnyNumbers(6, 45)
                slip = listToString(number)
                core.buy(slip)
                time.sleep(1.0)
        except:
            print 'closing safely...'
            core.close()
            
            time.sleep(0.5)
    except:
        print 'connect failed!'

if __name__ == '__main__' :
    __test__()

